﻿module Interpreter

open LambdaCalculus

type Scope = Global | Subscope of (string * Value) * Scope
and Value = Func of (Value -> Value)

let rec lookup varname scope = 
   match scope with
   | Global -> failwithf "Variable `%s` not in scope" varname
   | Subscope((key, value), parent) -> if key = varname then value else lookup varname parent

let rec eval term scope = 
    match term with
    | Variable(x) -> lookup x scope
    | Application(func, arg) -> 
        match eval func scope with
        | Func(f) -> f(eval arg scope)
    | Lambda(x, body) -> Func(fun arg -> eval body (Subscope((x,arg),scope)))

let dummy = Func(fun v -> v)

let dechurch v =
    let cnt = ref 0
    match v with
    | Func(f) -> 
        let curry = f(Func(fun arg -> cnt := !cnt + 1; arg)) in
        match curry with
        | Func(g) -> ignore (g(dummy)); !cnt